/* PYVMS_INIT.C -- init values -- 19-SEP-1997 Uwe Zessin

18-NOV-1997 ZE. -- add initialization of pyvms_gl_keyboard_id +
		   pyvms_gl_key_table_id
19-NOV-1997 ZE. -- add initialization of pyvms_gar_rlbuffer
30-NOV-1997 ZE. -- alter argv[0] from VMS to Unix notation
14-DEC-1997 ZE. -- Fix malloc() bug. Include SMG$ROUTINES.H.
01-NOV-1999 ZE. -- support for uname() - init structure
23-FEB-2000 ZE. -- initialize global cells for builtin timezone support
28-FEB-2000 ZE. -- replace spaces in string |vms_gr_utsname.machine| with '_'
30-APR-2000 ZE. -- changes to re-establish POSIX support
30-MAY-2000 ZE. -- S_HW_NAME appears to short for a XP1000
--------------------
$ @ PYTHON_VMS:BLDRUN  VMS  PYVMS_INIT

*/

#if defined(__DECC) || defined(__DECCXX)
#pragma module PYVMS_INIT "20000530"
#endif

/* ------------------------------------------------------------------------- */
#include <descrip.h>
#include <lib$routines.h>
#include <smg$routines.h>
#include <ssdef.h>
#include <starlet.h>
#include <syidef.h>

#include "vmsdef.h"
#include "pyvms_readline.h"
#include "vms__utsname.h"

#if !defined(__VMS_POSIX) && __VMS_VER < 70000000
void VMS__TZSET();
#endif

#include <unixio.h>
#include <unixlib.h>
#ifdef __DECC
/* DECC on Alpha does redefine these already */
#ifndef shell$from_vms
#define shell$from_vms(_p1_,_p2_,_p3_) decc$from_vms(_p1_,_p2_,_p3_)
#endif
#ifndef shell$translate_vms
#define shell$translate_vms(_p1_) decc$translate_vms(_p1_)
#endif
#ifndef shell$to_vms
#define shell$to_vms(_p1_,_p2_,_p3_,_p4_,_p5_) decc$to_vms(_p1_,_p2_,_p3_,_p4_,_p5_)
#endif
#endif /* __DECC */

/* ------------------------------------------------------------------------- */

/* numeric value of current VMS version (PYVMS_GLOBAL.C) */
extern vmsdef_xw_vmsver vmsdef_gw_vmsver;

/* VMS version string to numeric code translation table (VMSDAT_VMSVER.MAR) */
extern struct vmsdef_xr_vmsvertbl VMSDEF_GR_VMSDAT_VMSVER[];

/* readline / SMG$ support */
extern long			  pyvms_gl_keyboard_id;
extern long			  pyvms_gl_key_table_id;
extern struct pyvms_xr_rlbuffer	* pyvms_gar_rlbuffer;	/* listhead */

/* flags for tracing */
extern long			  pyvms_gl_trace_open;
extern long			  pyvms_gl_trace_stat;

/* uname() support */

extern struct utsname  vms_gr_utsname;
extern char          * vms_gt_openvms;
extern char          * vms_gt_alpha;       /* architectures */
extern char          * vms_gt_vax;

extern char	     * vms_gat_tzname [2];
extern int	       vms_gl_daylight;
extern long	       vms_gl_timezone;

extern long	       vms_gl_dst_offset;

extern long	       vms_gl_tzdiff;

/* ------------------------------------------------------------------------- */

static char     pvi_gt_psxpath[1026];
static char   * pvi_gat_psxpath;

static int pvi_from_vms_action (char *spec)
{
    (void)strcpy (pvi_gat_psxpath, spec);
    return 1;
}

/* ---------------------------------------- */

int PyVMS_init (int *pvi_argc, char ***pvi_argv)
{
#define S_VMSVER    8
#define S_NODENAME 15
#define S_HW_NAME  63
    struct dsc$descriptor_s	r_vmsver;

    char			t_vmsver[S_VMSVER+1];
    unsigned short int		w_vmsver_len;
    char			t_nodename[S_NODENAME+1];
    unsigned short int		w_nodename_len;
    char			t_hw_name[S_HW_NAME+1];
    unsigned short int		w_hw_name_len;

    unsigned long		l_item_code;
    unsigned long		l_system_id;
    unsigned long		l_status;
    unsigned long		l_cpu;
    unsigned long		l_ver_index;

    int			argc   = *pvi_argc;	/* Argument Count  */
    char		**argv = *pvi_argv;	/* Argument Vector */
    unsigned long	l_argv0_len;
    char		*at_argv0;

    /* -------------------- */
    /* clear |vms_gr_utsname| structure */
    (void) memset (&vms_gr_utsname, '\0', sizeof(vms_gr_utsname) );

    /* -------------------- */
    /* copy OS name ('OpenVMS') in |vms_gr_utsname| structure */
    (void) strncpy (vms_gr_utsname.sysname
		   ,vms_gt_openvms
		   ,sizeof(vms_gr_utsname.sysname) - 1
		   );

    /* -------------------- */
    /* build string descriptor for resultant-string argument */
    r_vmsver.dsc$w_length  = S_VMSVER;
    r_vmsver.dsc$b_dtype   = DSC$K_DTYPE_T;
    r_vmsver.dsc$b_class   = DSC$K_CLASS_S;
    r_vmsver.dsc$a_pointer = &t_vmsver[0];

    /* get VMS version as a string */
    l_item_code = SYI$_VERSION;
    l_system_id = 0;			/* current system */
    l_status = lib$getsyi
		(&l_item_code, 0, &r_vmsver, &w_vmsver_len, &l_system_id, 0);
    if (l_status != SS$_NORMAL)
    {
	printf ("%%PyVMS_init(): LIB$GETSYI(SYI$_VERSION) failed\n");
	lib$stop (l_status);
    }

    /* terminate version string for strcmp() */
    t_vmsver[w_vmsver_len] = '\0';

    /* copy OS version in |vms_gr_utsname| structure */
    (void) strncpy (vms_gr_utsname.version
		   ,t_vmsver
		   ,sizeof(vms_gr_utsname.version) - 1
		   );

    /* -------------------- */
    /* drop trailing spaces in |vms_gr_utsname.version| */
    {
	char * cp;

	cp = (char *)&vms_gr_utsname.version;
	cp += (int)w_vmsver_len;
	cp--;		/* last character in string */
	while (*cp == ' ')
	{
	    *cp = '\0';	/* delete a trailing space */
	    cp--;	/* previous character */
	}
    }

    /* -------------------- */
    /* get CPU number to destinguish between VAX and ALPHA */
    l_item_code = SYI$_CPU;
    l_system_id = 0;			/* current system */
    l_status = lib$getsyi
		(&l_item_code, &l_cpu, 0, 0, &l_system_id, 0);
    if (l_status != SS$_NORMAL)
    {
	printf ("%%PyVMS_init(): LIB$GETSYI(SYI$_CPU) failed\n");
	lib$stop (l_status);
    }

    if (l_cpu < 128)
    {
	l_cpu = ITMTBL_M_ARCH_VAX;	/* CPU is a VAX */

	/* copy architecture name in |vms_gr_utsname| structure */
	(void) strncpy (vms_gr_utsname.arch
		   ,vms_gt_vax
		   ,3 /* sizeof(vms_gr_utsname.arch) - 1 */
		   );
    }
    else
    {
	l_cpu = ITMTBL_M_ARCH_ALPHA;	/* CPU is an Alpha */

	/* copy architecture name in utsname structure */
	(void) strncpy (vms_gr_utsname.arch
		   ,vms_gt_alpha
		   ,5 /* sizeof(vms_gr_utsname.arch) - 1 */
		   );
    }

    /* -------------------- */
    /* translate from text string to internal version number */
    vmsdef_gw_vmsver = 0;
    l_ver_index = 0;
    while (VMSDEF_GR_VMSDAT_VMSVER[l_ver_index].w_vmsver != 0)
    {
        /* check VMS version */
        if (strcmp(t_vmsver,VMSDEF_GR_VMSDAT_VMSVER[l_ver_index].at_vmsver)==0)
        {
	  /* found textual match - check if valid for VAX or Alpha */
	  if ((l_cpu && ITMTBL_M_ARCH_VAX) || (l_cpu && ITMTBL_M_ARCH_ALPHA))
          {
            /* found textual match - use numerical value of item-code */
	    vmsdef_gw_vmsver = VMSDEF_GR_VMSDAT_VMSVER[l_ver_index].w_vmsver;
	    break;
          }
	}
	l_ver_index++;			/* check next version string */
    }

    if (VMSDEF_GR_VMSDAT_VMSVER[l_ver_index].w_vmsver == 0)
    {
	printf (
    "%%PyVMS_init(): version string not found in VMSDEF_GR_VMSDAT_VMSVER\n");
	printf (" \\%s\\\n", t_vmsver);
	lib$stop (SS$_ABORT);
    }

    /* -------------------- */
    /* get SYI$_HW_NAME for |vms_gr_utsname| */

    /* build string descriptor for resultant-string argument */
    r_vmsver.dsc$w_length  = S_HW_NAME;
    r_vmsver.dsc$b_dtype   = DSC$K_DTYPE_T;
    r_vmsver.dsc$b_class   = DSC$K_CLASS_S;
    r_vmsver.dsc$a_pointer = &t_hw_name[0];

    /* get HW_NAME as a string */
    l_item_code = SYI$_HW_NAME;
    l_system_id = 0;			/* current system */
    l_status = lib$getsyi
		(&l_item_code, 0, &r_vmsver, &w_hw_name_len, &l_system_id, 0);
    if (l_status != SS$_NORMAL)
    {
	printf ("%%PyVMS_init(): LIB$GETSYI(SYI$_HW_NAME) failed\n");
	lib$stop (l_status);
    }

    /* copy HW_NAME in |vms_gr_utsname| structure */
    (void) strncpy (vms_gr_utsname.machine
		   ,t_hw_name
		   ,sizeof(vms_gr_utsname.machine) - 1
		   );

    /* -------------------- */
    /* drop trailing spaces in |vms_gr_utsname.machine| */
    {
	char * cp;

	cp = (char *)&vms_gr_utsname.machine;
	cp += (int)w_hw_name_len;
	*cp = '\0';	/* set string end */
    }

    /* -------------------- */
    /* replace spaces in string with '_' */
    {
	char * cp;

	cp = (char *)&vms_gr_utsname.machine;
	while (*cp != '\0')
	{
	    if (*cp == ' ')
	    {
		*cp = '_';	/* replace character */
	    }
	    cp++;		/* next character */
	}
    }

    /* -------------------- */
#ifndef __VMS_POSIX
    /* initialize virtual keyboard for PYVMS_READLINE.C */
    l_status = smg$create_key_table (&pyvms_gl_key_table_id);
    if (l_status != SS$_NORMAL)
    {
	printf ("%%PyVMS_init(): SMG$CREATE_KEY_TABLE() failed\n");
	lib$stop (l_status);
    }

    l_status = smg$create_virtual_keyboard (&pyvms_gl_keyboard_id);
    if (l_status != SS$_NORMAL)
    {
	printf ("%%PyVMS_init(): SMG$CREATE_VIRTUAL_KEYBOARD() failed\n");
	lib$stop (l_status);
    }
#endif /* !__VMS_POSIX */

    /* mark listhead for recall buffer as empty */
    pyvms_gar_rlbuffer = NULL;

    /* -------------------- */
    /* build string descriptor for resultant-string argument */
    /* re-use descriptor */
    r_vmsver.dsc$w_length  = S_NODENAME;
    r_vmsver.dsc$b_dtype   = DSC$K_DTYPE_T;
    r_vmsver.dsc$b_class   = DSC$K_CLASS_S;
    r_vmsver.dsc$a_pointer = &t_nodename[0];

    /* get VMS nodename */
    l_item_code = SYI$_NODENAME;
    l_system_id = 0;			/* current system */
    l_status = lib$getsyi
		(&l_item_code, 0, &r_vmsver, &w_nodename_len, &l_system_id, 0);
    if (l_status != SS$_NORMAL)
    {
	printf ("%%PyVMS_init(): LIB$GETSYI(SYI$_NODENAME) failed\n");
	lib$stop (l_status);
    }

    /* terminate nodename string for strncpy() */
    t_nodename[w_nodename_len] = '\0';

    /* copy nodename in utsname structure */
    (void) strncpy (vms_gr_utsname.nodename
		   ,t_nodename
		   ,sizeof(vms_gr_utsname.nodename) - 1
		   );

    /* -------------------- */
    /* translate path in argv[0] from VMS to Unix notation */
#ifndef __VMS_POSIX
    pvi_gat_psxpath = &pvi_gt_psxpath[0];
    (void)shell$from_vms (argv[0],pvi_from_vms_action,0);
	/* 0 = no wildcard expansion */
    /* make a copy of the string - future versions might move the translation */
    /*  to a common module */
    l_argv0_len = strlen(pvi_gat_psxpath);
    at_argv0 = malloc(l_argv0_len+1);
    if (at_argv0 == NULL)
    {
	printf ("%%PyVMS_init(): malloc(l_argv0_len+1) failed\n");
	lib$stop (4852); /* %SYSTEM-F-INSVIRMEM - def. severify = W ! */
    }
    /* copy translated buffer into new path */
    (void) strcpy (at_argv0, pvi_gat_psxpath);
    /* replace argument */
    argv[0] = at_argv0;
#endif /* !__VMS_POSIX */

    /* -------------------- */

    pyvms_gl_trace_open = 1;
    pyvms_gl_trace_stat = 1;

    /* ---------------------------------------- */
    /* initialize global cells for builtin timezone support */

    /* note: [vms__]tzset() will set it correcly */
    /*  this imitates Posix behaviour */
    vms_gat_tzname [0] = NULL;
    vms_gat_tzname [1] = NULL;
    vms_gl_daylight    = 0;
    vms_gl_timezone    = 0;

    vms_gl_dst_offset  = 0;

    vms_gl_tzdiff      = 0;

    /* ---------------------------------------- */
#if !defined(__VMS_POSIX) && __VMS_VER < 70000000
(void) VMS__TZSET ();
#endif
    /* ---------------------------------------- */

    return 0;
} /* int PyVMS_init(...) */

/* ------------------------------------------------------------------------- */

/* EOF: PYVMS_INIT.C */
