/* VMS_SMG_CREATE_VIRTUAL_KEYBOARD.C -- 06-MAR-2000 Uwe Zessin
   Python interface to SMG$CREATE_VIRTUAL_KEYBOARD

--------------------
$ @ PYTHON_VMS:BLDRUN  VMS  VMS_SMG_CREATE_VIRTUAL_KEYBOARD

*/

#if defined(__DECC) || defined(__DECCXX)
#pragma module VMS_SMG_CREATE_VIRTUAL_KEYBOARD "20000306"
#endif

/* ------------------------------------------------------------------------- */

#include "python.h"
#include "vmsdef.h"

#include <descrip.h>
#include <lib$routines.h>	/* LIB$name  */
#include <smg$routines.h>	/* SMG$name  */
#include <ssdef.h>		/* SS$_name  */

/* ------------------------------------------------------------------------- */
extern PyObject *vms_smg_gr_error;		/* exception vms_smg.error */
/* ------------------------------------------------------------------------- */

char vms_smg_create_virtual_kbd__doc[] =
"keyboard_id, resultant_filespec\
 = vms_smg.create_virtual_keyboard\
 ([input_device], [default_filespec], [recall_size])\n\
Create a Virtual Keyboard.";

/* ------------------------------------------------------------------------- */

PyObject *
vms_smg_create_virtual_keyboard (PyObject *self, PyObject *args)
{
	char			* at_input_device;
	struct dsc$descriptor_s   r_input_device;
	struct dsc$descriptor_s * ar_input_device;
	unsigned long		  l_input_device_len;

	char			* at_default_filespec;
	struct dsc$descriptor_s   r_default_filespec;
	struct dsc$descriptor_s * ar_default_filespec;
	unsigned long		  l_default_filespec_len;

	PyObject		* ar_recall_size;
	unsigned long		  l_recall_size;
        unsigned char		  b_recall_size;
	unsigned char		* ab_recall_size;

	unsigned long		  l_keyboard_id;
	struct dsc$descriptor_s   r_resultant_filespec;

	unsigned long		  l_status_smg;
	unsigned long		  l_status_free;

	PyObject		* ar_return;

	/* -------------------- */
	at_input_device     = NULL; l_input_device_len = 0;
	at_default_filespec = NULL; l_default_filespec_len = 0;
	ar_recall_size      = Py_None;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "|z#z#O",
	    &at_input_device,     &l_input_device_len,
	    &at_default_filespec, &l_default_filespec_len,
	    &ar_recall_size))
	{
	    return NULL;
	}

	/* -------------------- */
	/* argument 1: input_device */
	if (at_input_device == NULL)
	{
	    ar_input_device = 0;	/* omitted */
	}
	else
	{
	    if (l_input_device_len > 65535)
	    {
		PyErr_SetString(PyExc_ValueError,
	 "argument 1: input_device - string size limited to 65535 characters");
		return NULL;
	    }
	    r_input_device.dsc$w_length  = l_input_device_len;
	    r_input_device.dsc$b_dtype   = DSC$K_DTYPE_T;
	    r_input_device.dsc$b_class   = DSC$K_CLASS_S;
	    r_input_device.dsc$a_pointer = at_input_device;
	    ar_input_device = &r_input_device;
	}

	/* -------------------- */
	/* argument 2: default_filespec */
	if (at_default_filespec == NULL)
	{
	    ar_default_filespec = 0;	/* omitted */
	}
	else
	{
	    if (l_default_filespec_len > 65535)
	    {
		PyErr_SetString(PyExc_ValueError,
     "argument 2: default_filespec - string size limited to 65535 characters");
		return NULL;
	    }
	    r_default_filespec.dsc$w_length  = l_default_filespec_len;
	    r_default_filespec.dsc$b_dtype   = DSC$K_DTYPE_T;
	    r_default_filespec.dsc$b_class   = DSC$K_CLASS_S;
	    r_default_filespec.dsc$a_pointer = at_default_filespec;
	    ar_default_filespec = &r_default_filespec;
	}

	/* -------------------- */
	/* argument 3: [recall_size] */
	if (ar_recall_size == Py_None)
	{
	    ab_recall_size = 0;		/* omitted */
	    b_recall_size  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_recall_size))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 2: recall_size - must be integer or None");
		return NULL;
	    }
	    l_recall_size = PyInt_AsLong(ar_recall_size);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    if ((l_recall_size < 0) || (l_recall_size > 255))
	    {
		PyErr_SetString(PyExc_ValueError,
		    "argument 2: recall_size - must be between 0 and 255");
		return NULL;
	    }
	    b_recall_size  = l_recall_size;
	    ab_recall_size = &b_recall_size;
	}

	/* -------------------- */
	/* set up string descriptor */
	/*  let SMG$CREATE_VIRTUAL_KEYBOARD allocate the memory */
	r_resultant_filespec.dsc$w_length  = 0;
	r_resultant_filespec.dsc$b_dtype   = DSC$K_DTYPE_T;
	r_resultant_filespec.dsc$b_class   = DSC$K_CLASS_D; /* not _S !! */
	r_resultant_filespec.dsc$a_pointer = 0;

	/* -------------------- */
	l_status_smg = smg$create_virtual_keyboard
		(&l_keyboard_id
		,ar_input_device
		,ar_default_filespec
		,&r_resultant_filespec
		,ab_recall_size
		);

	/* -------------------- */
	if (l_status_smg == SS$_NORMAL)
	{
	    PyObject	* ar_strobj;

	    ar_strobj = Py_BuildValue
		("s#"
		,(r_resultant_filespec.dsc$w_length == 0) /* empty string? */
		  ? (void*)&r_resultant_filespec	  /* non-NULL address */
		  : (void*)r_resultant_filespec.dsc$a_pointer
		,(unsigned int)r_resultant_filespec.dsc$w_length
		);
	    if (ar_strobj != NULL)
	    {
		ar_return = Py_BuildValue ("iO", l_keyboard_id, ar_strobj);
		Py_DECREF(ar_strobj);
	    }
	    else
	    {
		ar_return = NULL;
	    }
	}
	else
	{
	    ar_return = NULL;
	}

	/* -------------------- */
	/* deallocate memory of dynamic string descriptor */
	l_status_free = lib$sfree1_dd (&r_resultant_filespec);
	if (l_status_free != SS$_NORMAL)
	{
	    PyErr_SetString(PyExc_SystemError,
		"vms_smg_create_virtual_keyboard: LIB$SFREE1_DD() failed");
	    (void) PyErr_Print();

	    if (l_status_smg == SS$_NORMAL)
	    {
		l_status_smg = l_status_free; /* fail anyway */
	        Py_XDECREF(ar_return);
	    }
	}

	/* -------------------- */
	if (l_status_smg == SS$_NORMAL)
	{
	    return ar_return;
	}

	Py_XDECREF(ar_return);

	/* -------------------- */
	/* error */
	return PyVMS_ErrSetVal(vms_smg_gr_error, 1, l_status_smg);
} /* vms_smg_create_virtual_keyboard () */

/* ------------------------------------------------------------------------- */

/* EOF: VMS_SMG_CREATE_VIRTUAL_KEYBOARD.C */
