/* VMS_SMG_GET_KEY_DEF.C -- 20-MAR-2000 Uwe Zessin
   Python interface to SMG$GET_KEY_DEF

--------------------
$ @ PYTHON_VMS:BLDRUN  VMS  VMS_SMG_GET_KEY_DEF

*/

#if defined(__DECC) || defined(__DECCXX)
#pragma module VMS_SMG_GET_KEY_DEF "20000320"
#endif

/* ------------------------------------------------------------------------- */

#include "vmsdef.h"		/* includes "Python.h" */

#include <descrip.h>
#include <lib$routines.h>	/* LIB$name  */
#include <smg$routines.h>	/* SMG$name  */
#include <ssdef.h>		/* SS$_name  */

/* ------------------------------------------------------------------------- */
extern PyObject *vms_smg_gr_error;		/* exception vms_smg.error */
/* ------------------------------------------------------------------------- */

char vms_smg_get_key_def__doc[] =
"attributes, equivalence_string, state_string\
 = vms_smg.get_key_def (key_table_id, key_name, [if_state])\n\
Get Key Definition.";

/* ------------------------------------------------------------------------- */

PyObject *
vms_smg_get_key_def (PyObject *self, PyObject *args)
{
	unsigned long		  l_key_table_id;

	char			* at_key_name;
	struct dsc$descriptor_s   r_key_name;
	struct dsc$descriptor_s * ar_key_name;
	unsigned long		  l_key_name_len;

	char			* at_if_state;
	struct dsc$descriptor_s   r_if_state;
	struct dsc$descriptor_s * ar_if_state;
	unsigned long		  l_if_state_len;

	unsigned long		  l_attributes;

	struct dsc$descriptor_s   r_equivalence_string;

	struct dsc$descriptor_s   r_state_string;

	unsigned long		  l_status_smg;
	unsigned long		  l_status_free;

	PyObject		* ar_return;

	/* -------------------- */
	at_if_state = NULL; l_if_state_len = 0;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "is#|z#"
	    ,&l_key_table_id
	    ,&at_key_name, &l_key_name_len
	    ,&at_if_state, &l_if_state_len
	    ))
	{
	    return NULL;
	}

	/* -------------------- */
	/* argument 1: key_table_id */

	/* -------------------- */
	/* argument 2: key_name */
	if (l_key_name_len > 65535)
	{
	    PyErr_SetString(PyExc_ValueError,
	     "argument 2: key_name - string size limited to 65535 characters");
	    return NULL;
	}
	r_key_name.dsc$w_length  = l_key_name_len;
	r_key_name.dsc$b_dtype   = DSC$K_DTYPE_T;
	r_key_name.dsc$b_class   = DSC$K_CLASS_S;
	r_key_name.dsc$a_pointer = at_key_name;
	ar_key_name = &r_key_name;

	/* -------------------- */
	/* argument 3: if_state */
	if (at_if_state == NULL)
	{
	    ar_if_state = 0;	/* omitted */
	}
	else
	{
	    if (l_if_state_len > 65535)
	    {
		PyErr_SetString(PyExc_ValueError,
	     "argument 3: if_state - string size limited to 65535 characters");
		return NULL;
	    }
	    r_if_state.dsc$w_length  = l_if_state_len;
	    r_if_state.dsc$b_dtype   = DSC$K_DTYPE_T;
	    r_if_state.dsc$b_class   = DSC$K_CLASS_S;
	    r_if_state.dsc$a_pointer = at_if_state;
	    ar_if_state = &r_if_state;
	}

	/* -------------------- */
	l_attributes = 0;

	/* set up string descriptors */
	/*  let SMG$GET_KEY_DEF allocate the memory */
	r_equivalence_string.dsc$w_length  = 0;
	r_equivalence_string.dsc$b_dtype   = DSC$K_DTYPE_T;
	r_equivalence_string.dsc$b_class   = DSC$K_CLASS_D; /* not _S !! */
	r_equivalence_string.dsc$a_pointer = 0;

	r_state_string.dsc$w_length  = 0;
	r_state_string.dsc$b_dtype   = DSC$K_DTYPE_T;
	r_state_string.dsc$b_class   = DSC$K_CLASS_D; /* not _S !! */
	r_state_string.dsc$a_pointer = 0;

	/* -------------------- */
	l_status_smg = smg$get_key_def
		(&l_key_table_id
		,r_key_name
		,ar_if_state
		,&l_attributes
		,&r_equivalence_string
		,&r_state_string
		);

	/* -------------------- */
	if (l_status_smg == SS$_NORMAL)
	{
	    PyObject	* ar_equivalence_string;
	    PyObject	* ar_state_string;

	    ar_equivalence_string = Py_BuildValue
		("s#"
		,(r_equivalence_string.dsc$w_length == 0) /* empty string? */
		  ? (void*)&r_equivalence_string       /* non-NULL address */
		  : (void*)r_equivalence_string.dsc$a_pointer
		,(unsigned int)r_equivalence_string.dsc$w_length
		);
	    if (ar_equivalence_string != NULL)
	    {
		ar_state_string = Py_BuildValue
		    ("s#"
		    ,(r_state_string.dsc$w_length == 0)	/* empty string? */
		      ? (void*)&r_state_string		/* non-NULL address */
		      : (void*)r_state_string.dsc$a_pointer
		    ,(unsigned int)r_state_string.dsc$w_length
		    );
		if (ar_state_string != NULL)
		{
		    ar_return = Py_BuildValue ("iOO"
			,l_attributes
			,ar_equivalence_string
			,ar_state_string
			);
		    Py_DECREF(ar_equivalence_string);
		    Py_DECREF(ar_state_string);
		} /* if (ar_state_string != NULL) */
		else
		{
		    Py_DECREF(ar_equivalence_string);
		    ar_return = NULL;
		}
	    } /* if (ar_equivalence_string != NULL) */
	    else
	    {
		ar_return = NULL;
	    }
	} /* if (l_status_smg == SS$_NORMAL) */
	else
	{
	    ar_return = NULL;
	}

	/* -------------------- */
	/* deallocate memory of dynamic string descriptors */
	l_status_free = lib$sfree1_dd (&r_equivalence_string);
	if (l_status_free != SS$_NORMAL)
	{
	    PyErr_SetString(PyExc_SystemError,
		"vms_smg_get_key_def: LIB$SFREE1_DD(r_equivalence_string) failed");
	    (void) PyErr_Print();

	    if (l_status_smg == SS$_NORMAL)
	    {
		l_status_smg = l_status_free; /* fail anyway */
	        Py_XDECREF(ar_return);
		ar_return = NULL;
	    }
	}
	/* ---------- */
	l_status_free = lib$sfree1_dd (&r_state_string);
	if (l_status_free != SS$_NORMAL)
	{
	    PyErr_SetString(PyExc_SystemError,
		"vms_smg_get_key_def: LIB$SFREE1_DD(r_state_string) failed");
	    (void) PyErr_Print();

	    /* if the previous free() failed, then l_status_smg will already */
	    /* have an error status set */
	    if (l_status_smg == SS$_NORMAL)
	    {
		l_status_smg = l_status_free; /* fail anyway */
	        Py_XDECREF(ar_return);
	    }
	}

	/* -------------------- */
	if (l_status_smg == SS$_NORMAL)
	{
	    return ar_return;
	}

	Py_XDECREF(ar_return);

	/* -------------------- */
	/* error */
	return PyVMS_ErrSetVal(vms_smg_gr_error, 1, l_status_smg);
} /* vms_smg_get_key_def () */

/* ------------------------------------------------------------------------- */

/* EOF: VMS_SMG_GET_KEY_DEF.C */
