/* VMS_SMG_GET_PASTEBOARD_ATTRIBUTES.C -- 21-MAR-2000 Uwe Zessin
   Python interface to SMG$GET_PASTEBOARD_ATTRIBUTES

--------------------
$ @ PYTHON_VMS:BLDRUN  VMS  VMS_SMG_GET_PASTEBOARD_ATTRIBUTES

*/

#if defined(__DECC) || defined(__DECCXX)
#pragma module VMS_SMG_GET_PASTEBOARD_ATTR "20000321"
#endif

/* ------------------------------------------------------------------------- */

#include "vmsdef.h"		/* includes "Python.h" */

#include <smg$routines.h>	/* SMG$name  */
#include <smgdef.h>
#include <ssdef.h>		/* SS$_name  */

/* ------------------------------------------------------------------------- */
extern PyObject *vms_smg_gr_error;		/* exception vms_smg.error */
/* ------------------------------------------------------------------------- */

char vms_smg_get_pastebrd_attr__doc[] =
"pasteboard_info_table = vms_smg.get_pasteboard_attributes (pasteboard_id)\n\
Get Pasteboard Attributes.";

/* ------------------------------------------------------------------------- */

PyObject *
vms_smg_get_pasteboard_attr (PyObject *self, PyObject *args)
{
	unsigned long		  l_pasteboard_id;

	struct smg$r_attribute_info_block r_pbd_info;

	unsigned long		  l_pbd_info_len;

	unsigned long		  l_status;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "i", &l_pasteboard_id))
	{
	    return NULL;
	}

	/* -------------------- */
	/* argument 1: pasteboard_id */

	/* -------------------- */
	l_pbd_info_len = SMG$C_PASTEBOARD_INFO_BLOCK;

	/* -------------------- */
	l_status = smg$get_pasteboard_attributes
		(&l_pasteboard_id
		,&r_pbd_info
		,&l_pbd_info_len
		);

	/* -------------------- */
	if (l_status == SS$_NORMAL)
	{
	    /* create dictionary */
	    PyObject		  * ar_dict;	/* dictionary to be returned */
	    PyObject		  * ar_dictobj;	/* temp. for insert into " */

	    ar_dict = PyDict_New();
	    if (ar_dict == NULL)
	    {
		return NULL;
	    }

	    /* -------------------- */
	    /* The offset names that are inserted into |ar_dict| have been taken
		from the OpenVMS VAX V6.1 documentation. The code uses the
		members in struct |smg$r_attribute_info_block|. Unfortunately
		the names do not match exactly and other structs like
		|smg$r_pasteboard_info_block| cannot be used (I don't know
		how.) */
	    /* -------------------- */
	    /* ---------- */
	    ar_dictobj = PyInt_FromLong((unsigned int)r_pbd_info.smg$l_dev_char);
	    if (ar_dictobj == NULL)
	    {
		Py_DECREF(ar_dict);
		return NULL;
	    }
	    if (PyDict_SetItemString(ar_dict, "L_DEVCHAR", ar_dictobj) < 0)
	    {
		Py_DECREF(ar_dict);
		return NULL;
	    }
	    Py_DECREF(ar_dictobj);

	    /* ---------- */
	    ar_dictobj = PyInt_FromLong((unsigned int)r_pbd_info.smg$l_dev_depend);
	    if (ar_dictobj == NULL)
	    {
		Py_DECREF(ar_dict);
		return NULL;
	    }
	    if (PyDict_SetItemString(ar_dict, "L_DEVDEPEND", ar_dictobj) < 0)
	    {
		Py_DECREF(ar_dict);
		return NULL;
	    }
	    Py_DECREF(ar_dictobj);

	    /* ---------- */
	    ar_dictobj = PyInt_FromLong((unsigned int)r_pbd_info.smg$l_dev_depend2);
	    if (ar_dictobj == NULL)
	    {
		Py_DECREF(ar_dict);
		return NULL;
	    }
	    if (PyDict_SetItemString(ar_dict, "L_DEVDEPEND2", ar_dictobj) < 0)
	    {
		Py_DECREF(ar_dict);
		return NULL;
	    }
	    Py_DECREF(ar_dictobj);

	    /* ---------- */
	    ar_dictobj = PyInt_FromLong((unsigned int)r_pbd_info.smg$b_dev_class);
	    if (ar_dictobj == NULL)
	    {
		Py_DECREF(ar_dict);
		return NULL;
	    }
	    if (PyDict_SetItemString(ar_dict, "B_DEVCLASS", ar_dictobj) < 0)
	    {
		Py_DECREF(ar_dict);
		return NULL;
	    }
	    Py_DECREF(ar_dictobj);

	    /* ---------- */
	    ar_dictobj = PyInt_FromLong((unsigned int)r_pbd_info.smg$b_pbd_type);
	    if (ar_dictobj == NULL)
	    {
		Py_DECREF(ar_dict);
		return NULL;
	    }
	    if (PyDict_SetItemString(ar_dict, "B_SMG_DEVTYPE", ar_dictobj) < 0)
	    {
		Py_DECREF(ar_dict);
		return NULL;
	    }
	    Py_DECREF(ar_dictobj);

	    /* ---------- */
	    /* values like TT$_VT100 are in $TTDEF */
	    ar_dictobj = PyInt_FromLong((unsigned int)r_pbd_info.smg$b_dev_type);
	    if (ar_dictobj == NULL)
	    {
		Py_DECREF(ar_dict);
		return NULL;
	    }
	    if (PyDict_SetItemString(ar_dict, "B_PHY_DEVTYPE", ar_dictobj) < 0)
	    {
		Py_DECREF(ar_dict);
		return NULL;
	    }
	    Py_DECREF(ar_dictobj);

	    /* ---------- */
	    ar_dictobj = PyInt_FromLong((unsigned int)r_pbd_info.smg$b_num_rows);
	    if (ar_dictobj == NULL)
	    {
		Py_DECREF(ar_dict);
		return NULL;
	    }
	    if (PyDict_SetItemString(ar_dict, "B_ROWS", ar_dictobj) < 0)
	    {
		Py_DECREF(ar_dict);
		return NULL;
	    }
	    Py_DECREF(ar_dictobj);

	    /* ---------- */
	    ar_dictobj = PyInt_FromLong((unsigned int)r_pbd_info.smg$w_num_columns);
	    if (ar_dictobj == NULL)
	    {
		Py_DECREF(ar_dict);
		return NULL;
	    }
	    if (PyDict_SetItemString(ar_dict, "W_WIDTH", ar_dictobj) < 0)
	    {
		Py_DECREF(ar_dict);
		return NULL;
	    }
	    Py_DECREF(ar_dictobj);

	    /* ---------- */
	    /* @@ color values are in $SMGDEF */
	    ar_dictobj = PyInt_FromLong((unsigned int)r_pbd_info.smg$b_pbd_color);
	    if (ar_dictobj == NULL)
	    {
		Py_DECREF(ar_dict);
		return NULL;
	    }
	    if (PyDict_SetItemString(ar_dict, "B_COLOR", ar_dictobj) < 0)
	    {
		Py_DECREF(ar_dict);
		return NULL;
	    }
	    Py_DECREF(ar_dictobj);

	    /* ---------- */
	    ar_dictobj = PyInt_FromLong((unsigned int)r_pbd_info.smg$b_dev_parity);
	    if (ar_dictobj == NULL)
	    {
		Py_DECREF(ar_dict);
		return NULL;
	    }
	    if (PyDict_SetItemString(ar_dict, "B_PARITY", ar_dictobj) < 0)
	    {
		Py_DECREF(ar_dict);
		return NULL;
	    }
	    Py_DECREF(ar_dictobj);

	    /* ---------- */
	    ar_dictobj = PyInt_FromLong((unsigned int)r_pbd_info.smg$w_dev_speed);
	    if (ar_dictobj == NULL)
	    {
		Py_DECREF(ar_dict);
		return NULL;
	    }
	    if (PyDict_SetItemString(ar_dict, "W_SPEED", ar_dictobj) < 0)
	    {
		Py_DECREF(ar_dict);
		return NULL;
	    }
	    Py_DECREF(ar_dictobj);

	    /* ---------- */
	    ar_dictobj = PyInt_FromLong((unsigned int)r_pbd_info.smg$w_dev_fill);
	    if (ar_dictobj == NULL)
	    {
		Py_DECREF(ar_dict);
		return NULL;
	    }
	    if (PyDict_SetItemString(ar_dict, "W_FILL", ar_dictobj) < 0)
	    {
		Py_DECREF(ar_dict);
		return NULL;
	    }
	    Py_DECREF(ar_dictobj);

	    /* ---------- */
	    /* |smg$r_pasteboard_info_block| says: smg$w_cursor_row */
	    /* documentation uses the name from smg$r_attribute_info_block */
	    ar_dictobj = PyInt_FromLong((unsigned int)r_pbd_info.smg$w_phys_cursor_row);
	    if (ar_dictobj == NULL)
	    {
		Py_DECREF(ar_dict);
		return NULL;
	    }
	    if (PyDict_SetItemString(ar_dict, "W_PHYS_CURSOR_ROW", ar_dictobj) < 0)
	    {
		Py_DECREF(ar_dict);
		return NULL;
	    }
	    Py_DECREF(ar_dictobj);

	    /* ---------- */
	    /* |smg$r_pasteboard_info_block| says: smg$w_cursor_col */
	    /* documentation uses the name from smg$r_attribute_info_block */
	    ar_dictobj = PyInt_FromLong((unsigned int)r_pbd_info.smg$w_phys_cursor_col);
	    if (ar_dictobj == NULL)
	    {
		Py_DECREF(ar_dict);
		return NULL;
	    }
	    if (PyDict_SetItemString(ar_dict, "W_PHYS_CURSOR_COL", ar_dictobj) < 0)
	    {
		Py_DECREF(ar_dict);
		return NULL;
	    }
	    Py_DECREF(ar_dictobj);

	    /* ---------- */
	    /* Display id of topmostdisplay that cursor is in. */
	    /* 0 if cursor is not in a virtual display */
	    ar_dictobj = PyInt_FromLong((unsigned int)r_pbd_info.smg$l_display_id);
	    if (ar_dictobj == NULL)
	    {
		Py_DECREF(ar_dict);
		return NULL;
	    }
	    if (PyDict_SetItemString(ar_dict, "L_CURSOR_DID", ar_dictobj) < 0)
	    {
		Py_DECREF(ar_dict);
		return NULL;
	    }
	    Py_DECREF(ar_dictobj);

	    /* ---------- */
	    return (ar_dict);
	}

	/* -------------------- */
	/* error */
	return PyVMS_ErrSetVal(vms_smg_gr_error, 1, l_status);
} /* vms_smg_get_pasteboard_attr () */

/* ------------------------------------------------------------------------- */

/* EOF: VMS_SMG_GET_PASTEBOARD_ATTRIBUTES.C */
