/* VMS_SMG_SET_TERM_CHARACTERISTICS.C -- 01-APR-2000 Uwe Zessin
   Python interface to SMG$SET_TERM_CHARACTERISTICS

--------------------
$ @ PYTHON_VMS:BLDRUN  VMS  VMS_SMG_SET_TERM_CHARACTERISTICS

*/

#if defined(__DECC) || defined(__DECCXX)
#pragma module VMS_SMG_SET_TERM_CHARACTERISTIC "20000401"
#endif

/* ------------------------------------------------------------------------- */

#include "vmsdef.h"		/* includes "Python.h" */

#include <smg$routines.h>	/* SMG$name */
#include <ssdef.h>		/* SS$_name */

/* ------------------------------------------------------------------------- */
extern PyObject *vms_smg_gr_error;		/* exception vms_smg.error */
/* ------------------------------------------------------------------------- */

char vms_smg_set_term_char__doc[] =
"old_characteristics1, old_characteristics2 = vms_smg.set_term_characteristics\
 (pasteboard_id, [on_characteristics1], [on_characteristics2],\
 [off_characteristics1], [off_characteristics2])\n\
Change Terminal Characteristics.";

/* ------------------------------------------------------------------------- */

PyObject *
vms_smg_set_term_characteristic (PyObject *self, PyObject *args)
{
	unsigned long		  l_pasteboard_id;

	PyObject		* ar_on_characteristics1;
	unsigned long		  l_on_characteristics1;
	unsigned long		* al_on_characteristics1;

	PyObject		* ar_off_characteristics1;
	unsigned long		  l_off_characteristics1;
	unsigned long		* al_off_characteristics1;

	PyObject		* ar_on_characteristics2;
	unsigned long		  l_on_characteristics2;
	unsigned long		* al_on_characteristics2;

	PyObject		* ar_off_characteristics2;
	unsigned long		  l_off_characteristics2;
	unsigned long		* al_off_characteristics2;

	unsigned long		  l_old_characteristics1;
	unsigned long		  l_old_characteristics2;

	unsigned long		  l_status;

	/* -------------------- */
	ar_on_characteristics1  = Py_None;
	ar_off_characteristics1 = Py_None;
	ar_on_characteristics2  = Py_None;
	ar_off_characteristics2 = Py_None;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "i|OOOO"
	    ,&l_pasteboard_id
	    ,&ar_on_characteristics1
	    ,&ar_off_characteristics1
	    ,&ar_on_characteristics2
	    ,&ar_off_characteristics2
	    ))
	{
	    return NULL;
	}

	/* -------------------- */
	/* argument 1: pasteboard_id */

	/* -------------------- */
	/* argument 2: [on_characteristics1] */
	if (ar_on_characteristics1 == Py_None)
	{
	    al_on_characteristics1 = 0;		/* omitted */
	    l_on_characteristics1  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_on_characteristics1))
	    {
		PyErr_SetString(PyExc_TypeError,
		  "argument 2: on_characteristics1 - must be integer or None");
		return NULL;
	    }
	    l_on_characteristics1 = PyInt_AsLong(ar_on_characteristics1);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_on_characteristics1 = &l_on_characteristics1;
	}

	/* -------------------- */
	/* argument 3: [off_characteristics1] */
	if (ar_off_characteristics1 == Py_None)
	{
	    al_off_characteristics1 = 0;		/* omitted */
	    l_off_characteristics1  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_off_characteristics1))
	    {
		PyErr_SetString(PyExc_TypeError,
		 "argument 3: off_characteristics1 - must be integer or None");
		return NULL;
	    }
	    l_off_characteristics1 = PyInt_AsLong(ar_off_characteristics1);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_off_characteristics1 = &l_off_characteristics1;
	}

	/* -------------------- */
	/* argument 4: [on_characteristics2] */
	if (ar_on_characteristics2 == Py_None)
	{
	    al_on_characteristics2 = 0;		/* omitted */
	    l_on_characteristics2  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_on_characteristics2))
	    {
		PyErr_SetString(PyExc_TypeError,
		  "argument 4: on_characteristics2 - must be integer or None");
		return NULL;
	    }
	    l_on_characteristics2 = PyInt_AsLong(ar_on_characteristics2);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_on_characteristics2 = &l_on_characteristics2;
	}

	/* -------------------- */
	/* argument 5: [off_characteristics2] */
	if (ar_off_characteristics2 == Py_None)
	{
	    al_off_characteristics2 = 0;		/* omitted */
	    l_off_characteristics2  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_off_characteristics2))
	    {
		PyErr_SetString(PyExc_TypeError,
		 "argument 5: off_characteristics2 - must be integer or None");
		return NULL;
	    }
	    l_off_characteristics2 = PyInt_AsLong(ar_off_characteristics2);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_off_characteristics2 = &l_off_characteristics2;
	}

	/* -------------------- */
	l_status = smg$set_term_characteristics
		(&l_pasteboard_id
		,al_on_characteristics1
		,al_off_characteristics1
		,al_on_characteristics2
		,al_off_characteristics2
		,&l_old_characteristics1
		,&l_old_characteristics2
		);

	/* -------------------- */
	if (l_status == SS$_NORMAL)
	{
	    return Py_BuildValue ("ii"
		,l_old_characteristics1
		,l_old_characteristics2
		);
	}

	/* -------------------- */
	/* error */
	return PyVMS_ErrSetVal(vms_smg_gr_error, 1, l_status);
} /* vms_smg_set_term_characteristic () */

/* ------------------------------------------------------------------------- */

/* EOF: VMS_SMG_SET_TERM_CHARACTERISTICS.C */
