/* VMS_SYS_TIMCON.C -- 16-JAN-2000 Uwe Zessin
   Python interface to SYS$TIMCON()

13-DEC-1997 ZE. -- created in VMS_SYS.C
16-JAN-2000 ZE. -- extract into VMS_SYS_TIMCON.C for V5.5-2 exclusion
*/

#ifdef __DECC
#pragma module VMS_SYS_TIMCON "20000116"
#endif

#include "python.h"
#include "vmsdef.h"

#include <descrip.h>
#include <ssdef.h>		/* SS$_name	 */
#include <starlet.h>		/* SYS$name, ... */

/* ------------------------------------------------------------------------- */
extern PyObject *vms_sys_gr_error;	/* in VMS_SYS.C */
/* ------------------------------------------------------------------------- */

/* input a Python long integer - output a binary quadword */
long vms__cvt_pylong2quad (PyObject * r_longint, long * q_quadword);

/* ------------------------------ */
/* input a binary quadword - output a Python long integer */
PyObject * vms__cvt_quad2pylong (long q_quadword[2]);

/* ------------------------------ */
/* input a Python long integer, output a binary octaword */
extern long vms__cvt_pylong2octa (PyObject * r_longint, long * o_octaword);

/* ------------------------------ */
/* input a binary octaword, output a Python long integer */
extern PyObject * vms__cvt_octa2pylong (long q_octaword[4]);

/* ------------------------------------------------------------------------- */

char vms_sys_timcon__doc[] =
"utcadr = vms_sys.timcon ([timadr] , cvtflg=1)\n\
timadr = vms_sys.timcon ([utcadr] , cvtflg=0)\n\
Time Converter.";

PyObject *
vms_sys_timcon (PyObject *self, PyObject *args)
{
	long			  q_timadr[2];	/* binary system time */
	long			  o_utcadr[4];	/* utc_date_time      */
	long			* ax_argtim;

	/* ----- */
	PyObject		* ar_argtim;	/* argument 1	   */

	unsigned long		  l_cvtflg;	/* conversion flag $TIMCON */

	unsigned long		  l_status;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "Ol", &ar_argtim, &l_cvtflg))
	{
	    return NULL;
	}

	/* -------------------- */
	/* argument 1: argtim */
	if (ar_argtim == Py_None)
	{
	      ax_argtim = 0;		/* omitted */
	}
	else
	{
	  /* conversion direction doesn't matter - */
	  /*  must always be long integer */
	  if (!PyLong_Check(ar_argtim))
	  {
	      PyErr_SetString(PyExc_TypeError,
		"argument 1: argtim - must be long integer");
	      return NULL;
	  }
	  /* ---------- */
	  /* l_cvtflg = 0 -- convert 128bit utc to 64bit bintim */
	  if (l_cvtflg == 0)
	  {
	      /* convert Python long integer to binary octaword */
	      l_status = vms__cvt_pylong2octa (ar_argtim, &o_utcadr[0]);
	      if (l_status != SS$_NORMAL)
	      {
		  PyErr_SetString(PyExc_ValueError,
		   "argument 1: argtim - conversion from long integer failed");
		  return NULL;
	      }
	      ax_argtim = &o_utcadr[0];
	  } /* (l_cvtflg == 0) */
	  else
	  /* l_cvtflg = 1 -- convert 64bit bintim to 128bit utc */
	  /*	ar_argtim must be Python Long-Integer ! */
	  {
	    /* convert Python long integer to binary quadword */
	    l_status = vms__cvt_pylong2quad (ar_argtim, &q_timadr[0]);
	    if (l_status == -1)
	    {
	      return NULL;		/* error string has been set */
	    }
	    ax_argtim = &q_timadr[0];
	  } /* if (l_cvtflg == 0) */
	} /* if (ar_argtim == Py_None) */

	/* -------------------- */
	/* l_cvtflg = 0 -- convert 128bit utc to 64bit bintim */
	if (l_cvtflg == 0)
	{
	    l_status = sys$timcon
		(&q_timadr[0]		/* [timadr] */
		,ax_argtim		/* [utcadr] */
		,l_cvtflg		/* cvtflg   */
		);
	    if (l_status == SS$_NORMAL)
	    {
		/* convert binary quadword to Python long integer */
		return vms__cvt_quad2pylong (&q_timadr[0]);
	    }
	}
	else
	{
	    l_status = sys$timcon
		(ax_argtim		/* [timadr] */
		,&o_utcadr[0]		/* [utcadr] */
		,l_cvtflg		/* cvtflg   */
		);
	    if (l_status == SS$_NORMAL)
	    {
		/* convert binary octaword to Python long integer */
		return vms__cvt_octa2pylong (&o_utcadr[0]);
	    }
	} /* if (l_cvtflg == 0) */

	/* -------------------- */
	/* error */
	return PyVMS_ErrSetVal(vms_sys_gr_error, 1, l_status);
} /* vms_sys_timcon () */

/* ------------------------------------------------------------------------- */

/* EOF: VMS_SYS_TIMCON.C */
